@@ -1,7 +1,13 @@
-/* BGP Extended Communities Attribute
-   Copyright (C) 2000 Kunihiro Ishiguro <kunihiro@zebra.org>
+/*
+    pmacct (Promiscuous mode IP Accounting package)
+    pmacct is Copyright (C) 2003-2012 by Paolo Lucente
+*/
 
-This file is part of pmacct but mostly based on GNU Zebra.
+/*
+ Originally based on Quagga BGP extended community attribute related
+ functions which is:
+
+ Copyright (C) 2000 Kunihiro Ishiguro <kunihiro@zebra.org>
 
 GNU Zebra is free software; you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
