@@ -1,7 +1,12 @@
-/* BGP routing table
-   Copyright (C) 1998, 2001 Kunihiro Ishiguro
+/*
+    pmacct (Promiscuous mode IP Accounting package)
+    pmacct is Copyright (C) 2003-2012 by Paolo Lucente
+*/
 
-This file is part of pmacct but mostly based on GNU Zebra.
+/* 
+ Originally based on Quagga BGP routing table which is:
+
+ Copyright (C) 1998, 2001 Kunihiro Ishiguro
 
 GNU Zebra is free software; you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
@@ -76,12 +81,19 @@ struct bgp_node
 */
 };
 
+struct bgp_info_extra
+{
+  rd_t rd;
+  u_char label[3];
+};
+
 struct bgp_info
 {
   struct bgp_info *next;
   struct bgp_info *prev;
   struct bgp_peer *peer;
   struct bgp_attr *attr;
+  struct bgp_info_extra *extra;
 };
 
 /* Prototypes */
