@@ -1,6 +1,6 @@
 /*
     pmacct (Promiscuous mode IP Accounting package)
-    pmacct is Copyright (C) 2003-2010 by Paolo Lucente
+    pmacct is Copyright (C) 2003-2012 by Paolo Lucente
 */
 
 /*
@@ -43,6 +43,8 @@ EXT int cfg_key_files_uid(char *, char *, char *);
 EXT int cfg_key_files_gid(char *, char *, char *);
 EXT int cfg_key_savefile_wait(char *, char *, char *);
 EXT int cfg_key_promisc(char *, char *, char *);
+EXT int cfg_key_num_protos(char *, char *, char *);
+EXT int cfg_key_num_hosts(char *, char *, char *);
 EXT int cfg_key_imt_path(char *, char *, char *);
 EXT int cfg_key_imt_passwd(char *, char *, char *);
 EXT int cfg_key_imt_buckets(char *, char *, char *);
@@ -76,7 +78,9 @@ EXT int cfg_key_sql_multi_values(char *, char *, char *);
 EXT int cfg_key_sql_aggressive_classification(char *, char *, char *);
 EXT int cfg_key_sql_locking_style(char *, char *, char *);
 EXT int cfg_key_sql_use_copy(char *, char *, char *);
+EXT int cfg_key_sql_delimiter(char *, char *, char *);
 EXT int cfg_key_plugin_pipe_size(char *, char *, char *);
+EXT int cfg_key_plugin_pipe_backlog(char *, char *, char *);
 EXT int cfg_key_plugin_buffer_size(char *, char *, char *);
 EXT int cfg_key_networks_mask(char *, char *, char *);
 EXT int cfg_key_networks_file(char *, char *, char *);
@@ -112,6 +116,7 @@ EXT int cfg_key_pre_tag_filter(char *, char *, char *);
 EXT int cfg_key_pre_tag2_filter(char *, char *, char *);
 EXT int cfg_key_post_tag(char *, char *, char *);
 EXT int cfg_key_sampling_rate(char *, char *, char *);
+EXT int cfg_key_sampling_map(char *, char *, char *);
 EXT int cfg_key_classifiers(char *, char *, char *);
 EXT int cfg_key_classifier_tentatives(char *, char *, char *);
 EXT int cfg_key_classifier_table_num(char *, char *, char *);
@@ -152,13 +157,19 @@ EXT int cfg_key_nfacctd_bgp_peer_as_skip_subas(char *, char *, char *);
 EXT int cfg_key_nfacctd_bgp_peer_src_as_map(char *, char *, char *);
 EXT int cfg_key_nfacctd_bgp_src_local_pref_map(char *, char *, char *);
 EXT int cfg_key_nfacctd_bgp_src_med_map(char *, char *, char *);
-EXT int cfg_key_nfacctd_bgp_is_symmetric_map(char *, char *, char *);
 EXT int cfg_key_nfacctd_bgp_to_agent_map(char *, char *, char *);
+EXT int cfg_key_nfacctd_bgp_iface_to_rd_map(char *, char *, char *);
 EXT int cfg_key_nfacctd_bgp_follow_default(char *, char *, char *);
 EXT int cfg_key_nfacctd_bgp_follow_nexthop(char *, char *, char *);
 EXT int cfg_key_nfacctd_bgp_neighbors_file(char *, char *, char *);
 EXT int cfg_key_nfacctd_bgp_md5_file(char *, char *, char *);
 EXT int cfg_key_nfacctd_bgp_table_peer_buckets(char *, char *, char *);
+EXT int cfg_key_nfacctd_isis(char *, char *, char *);
+EXT int cfg_key_nfacctd_isis_ip(char *, char *, char *);
+EXT int cfg_key_nfacctd_isis_net(char *, char *, char *);
+EXT int cfg_key_nfacctd_isis_iface(char *, char *, char *);
+EXT int cfg_key_nfacctd_isis_mtu(char *, char *, char *);
+EXT int cfg_key_nfacctd_isis_msglog(char *, char *, char *);
 EXT int cfg_key_uacctd_group(char *, char *, char *);
 EXT int cfg_key_uacctd_nl_size(char *, char *, char *);
 EXT int cfg_key_tunnel_0(char *, char *, char *);
