@@ -1,6 +1,6 @@
 /*
     pmacct (Promiscuous mode IP Accounting package)
-    pmacct is Copyright (C) 2003-2010 by Paolo Lucente
+    pmacct is Copyright (C) 2003-2011 by Paolo Lucente
 */
 
 /*
@@ -50,12 +50,15 @@ void pgsql_plugin(int pipe_fd, struct configuration *cfgptr, void *ptr)
   int pollagain = TRUE;
   u_int32_t seq = 1, rg_err_count = 0;
 
-  /* XXX: glue */
   memcpy(&config, cfgptr, sizeof(struct configuration));
   recollect_pipe_memory(ptr);
   pm_setproctitle("%s [%s]", "PostgreSQL Plugin", config.name);
   memset(&idata, 0, sizeof(idata));
   if (config.pidfile) write_pid_file_plugin(config.pidfile, config.type, config.name);
+  if (config.logfile) {
+    fclose(config.logfile_fd);
+    config.logfile_fd = open_logfile(config.logfile);
+  }
 
   sql_set_signals();
   sql_init_default_values();
@@ -102,6 +105,19 @@ void pgsql_plugin(int pipe_fd, struct configuration *cfgptr, void *ptr)
     idata.now = time(NULL);
     now = idata.now;
 
+    if (config.sql_history) {
+      while (idata.now > (idata.basetime + idata.timeslot)) {
+        time_t saved_basetime = idata.basetime;
+
+        idata.basetime += idata.timeslot;
+        if (config.sql_history == COUNT_MONTHLY)
+          idata.timeslot = calc_monthly_timeslot(idata.basetime, config.sql_history_howmany, ADD);
+        glob_basetime = idata.basetime;
+        idata.new_basetime = saved_basetime;
+        glob_new_basetime = saved_basetime;
+      }
+    }
+
     switch (ret) {
     case 0: /* poll(): timeout */
       if (qq_ptr) sql_cache_flush(queries_queue, qq_ptr, &idata, FALSE);
@@ -243,18 +259,6 @@ void pgsql_plugin(int pipe_fd, struct configuration *cfgptr, void *ptr)
       }
 
       data = (struct pkt_data *) (pipebuf+sizeof(struct ch_buf_hdr));
-      if (config.sql_history) {
-        while (idata.now > (idata.basetime + idata.timeslot)) {
-	  time_t saved_basetime = idata.basetime;
-
-	  idata.basetime += idata.timeslot;
-	  if (config.sql_history == COUNT_MONTHLY)
-	    idata.timeslot = calc_monthly_timeslot(idata.basetime, config.sql_history_howmany, ADD);
-	  glob_basetime = idata.basetime;
-	  idata.new_basetime = saved_basetime;
-	  glob_new_basetime = saved_basetime;
-	}
-      }
 
       while (((struct ch_buf_hdr *)pipebuf)->num) {
 	for (num = 0; net_funcs[num]; num++)
@@ -286,10 +290,16 @@ int PG_cache_dbop_copy(struct DBdesc *db, struct db_cache *cache_elem, struct in
 {
   PGresult *ret;
   char *ptr_values, *ptr_where;
+  char default_delim[] = ",", delim_buf[SRVBUFLEN];
   int num=0, have_flows=0;
 
   if (config.what_to_count & COUNT_FLOWS) have_flows = TRUE;
 
+  if (!config.sql_delimiter)
+    snprintf(delim_buf, SRVBUFLEN, "%s", default_delim);
+  else
+    snprintf(delim_buf, SRVBUFLEN, "%s", config.sql_delimiter);
+
   /* constructing SQL query */
   ptr_where = where_clause;
   ptr_values = values_clause;
@@ -303,11 +313,17 @@ int PG_cache_dbop_copy(struct DBdesc *db, struct db_cache *cache_elem, struct in
   }
 
 #if defined HAVE_64BIT_COUNTERS
-  if (have_flows) snprintf(ptr_values, SPACELEFT(values_clause), ",%llu,%llu,%llu\n", cache_elem->packet_counter, cache_elem->bytes_counter, cache_elem->flows_counter);
-  else snprintf(ptr_values, SPACELEFT(values_clause), ",%llu,%llu\n", cache_elem->packet_counter, cache_elem->bytes_counter);
+  if (have_flows) snprintf(ptr_values, SPACELEFT(values_clause), "%s%llu%s%llu%s%llu\n", delim_buf, cache_elem->packet_counter,
+											delim_buf, cache_elem->bytes_counter,
+											delim_buf, cache_elem->flows_counter);
+  else snprintf(ptr_values, SPACELEFT(values_clause), "%s%llu%s%llu\n", delim_buf, cache_elem->packet_counter,
+									delim_buf, cache_elem->bytes_counter);
 #else
-  if (have_flows) snprintf(ptr_values, SPACELEFT(values_clause), ",%lu,%lu,%lu\n", cache_elem->packet_counter, cache_elem->bytes_counter, cache_elem->flows_counter);
-  else snprintf(ptr_values, SPACELEFT(values_clause), ",%lu,%lu\n", cache_elem->packet_counter, cache_elem->bytes_counter);
+  if (have_flows) snprintf(ptr_values, SPACELEFT(values_clause), "%s%lu%s%lu%s%lu\n", delim_buf, cache_elem->packet_counter,
+											delim_buf, cache_elem->bytes_counter,
+											delim_buf, cache_elem->flows_counter);
+  else snprintf(ptr_values, SPACELEFT(values_clause), "%s%lu%s%lu\n", delim_buf, cache_elem->packet_counter,
+									delim_buf, cache_elem->bytes_counter);
 #endif
   strncpy(sql_data, values_clause, sizeof(sql_data));
   
@@ -425,13 +441,14 @@ void PG_cache_purge(struct db_cache *queue[], int index, struct insert_data *ida
   /* We check for variable substitution in SQL table */
   if (idata->dyn_table) {
     char tmpbuf[LONGLONGSRVBUFLEN];
+    time_t stamp = idata->new_basetime ? idata->new_basetime : idata->basetime;
 
-    strftime_same(copy_clause, LONGSRVBUFLEN, tmpbuf, &idata->basetime);
-    strftime_same(insert_clause, LONGSRVBUFLEN, tmpbuf, &idata->basetime);
-    strftime_same(update_clause, LONGSRVBUFLEN, tmpbuf, &idata->basetime);
-    strftime_same(lock_clause, LONGSRVBUFLEN, tmpbuf, &idata->basetime);
+    strftime_same(copy_clause, LONGSRVBUFLEN, tmpbuf, &stamp);
+    strftime_same(insert_clause, LONGSRVBUFLEN, tmpbuf, &stamp);
+    strftime_same(update_clause, LONGSRVBUFLEN, tmpbuf, &stamp);
+    strftime_same(lock_clause, LONGSRVBUFLEN, tmpbuf, &stamp);
 
-    if (config.sql_table_schema && idata->new_basetime) sql_create_table(bed.p, idata); 
+    if (config.sql_table_schema) sql_create_table(bed.p, &stamp); 
   }
   // strncat(update_clause, set_clause, SPACELEFT(update_clause));
 
@@ -523,12 +540,23 @@ int PG_evaluate_history(int primitive)
     strncat(copy_clause, "stamp_updated, stamp_inserted", SPACELEFT(copy_clause));
     strncat(insert_clause, "stamp_updated, stamp_inserted", SPACELEFT(insert_clause));
     if (config.sql_use_copy) {
+      char default_delim[] = ",", delim_buf[SRVBUFLEN];
+
+      if (!config.sql_delimiter || !config.sql_use_copy)
+        snprintf(delim_buf, SRVBUFLEN, "%s ", default_delim);
+      else
+        snprintf(delim_buf, SRVBUFLEN, "%s ", config.sql_delimiter);
+
       if (!config.sql_history_since_epoch) { 
-	strncat(values[primitive].string, "%s, %s", SPACELEFT(values[primitive].string));
+	strncat(values[primitive].string, "%s", SPACELEFT(values[primitive].string));
+	strncat(values[primitive].string, delim_buf, SPACELEFT(values[primitive].string));
+	strncat(values[primitive].string, "%s", SPACELEFT(values[primitive].string));
         values[primitive].handler = where[primitive].handler = count_copy_timestamp_handler;
       }
       else {
-	strncat(values[primitive].string, "%u, %u", SPACELEFT(values[primitive].string));
+	strncat(values[primitive].string, "%u", SPACELEFT(values[primitive].string));
+	strncat(values[primitive].string, delim_buf, SPACELEFT(values[primitive].string));
+	strncat(values[primitive].string, "%u", SPACELEFT(values[primitive].string));
         values[primitive].handler = where[primitive].handler = count_timestamp_handler;
       }
     }
@@ -550,6 +578,7 @@ int PG_evaluate_history(int primitive)
 int PG_compose_static_queries()
 {
   int primitives=0, set_primitives=0, have_flows=0, lock=0;
+  char default_delim[] = ",", delim_buf[SRVBUFLEN];
 
   if (config.what_to_count & COUNT_FLOWS || (config.sql_table_version >= 4 &&
                                              config.sql_table_version < SQL_TABLE_VERSION_BGP &&
@@ -573,7 +602,12 @@ int PG_compose_static_queries()
 
   strncat(copy_clause, ", packets, bytes", SPACELEFT(copy_clause));
   if (have_flows) strncat(copy_clause, ", flows", SPACELEFT(copy_clause));
-  strncat(copy_clause, ") FROM STDIN DELIMITER \',\'", SPACELEFT(copy_clause));
+
+  if (!config.sql_delimiter || !config.sql_use_copy)
+    snprintf(delim_buf, SRVBUFLEN, ") FROM STDIN DELIMITER \'%s\'", default_delim);
+  else
+    snprintf(delim_buf, SRVBUFLEN, ") FROM STDIN DELIMITER \'%s\'", config.sql_delimiter);
+  strncat(copy_clause, delim_buf, SPACELEFT(copy_clause));
 
   strncat(insert_clause, ", packets, bytes", SPACELEFT(insert_clause));
   if (have_flows) strncat(insert_clause, ", flows", SPACELEFT(insert_clause));
