@@ -1,6 +1,6 @@
 /*
     pmacct (Promiscuous mode IP Accounting package)
-    pmacct is Copyright (C) 2003-2009 by Paolo Lucente
+    pmacct is Copyright (C) 2003-2011 by Paolo Lucente
 */
 
 /*
@@ -786,3 +786,9 @@ void MY_exit_gracefully(int signum)
   printf("\nOK: written [%u/%u] elements.\n", we, re);
   exit(0);
 }
+
+/* Dummy version of bgp_rd2str() to resolve code dependencies */
+int bgp_rd2str(u_char *str, rd_t *rd)
+{
+  return TRUE;
+}
