@@ -1,6 +1,6 @@
 /*
     pmacct (Promiscuous mode IP Accounting package)
-    pmacct is Copyright (C) 2003-2010 by Paolo Lucente
+    pmacct is Copyright (C) 2003-2012 by Paolo Lucente
 */
 
 /*
@@ -33,6 +33,7 @@ const struct _map_dictionary_line tag_map_dictionary[] = {
   {"v8agg", PT_map_v8agg_handler},
   {"agent_id", PT_map_agent_id_handler},
   {"sampling_rate", PT_map_sampling_rate_handler},
+  {"sample_type", PT_map_sample_type_handler},
   {"direction", PT_map_direction_handler},
   {"src_as", PT_map_src_as_handler},
   {"dst_as", PT_map_dst_as_handler},
@@ -42,6 +43,7 @@ const struct _map_dictionary_line tag_map_dictionary[] = {
   {"local_pref", PT_map_local_pref_handler},
   {"src_comms", PT_map_src_comms_handler},
   {"comms", PT_map_comms_handler},
+  {"mpls_vpn_rd", PT_map_mpls_vpn_rd_handler},
   {"label", PT_map_label_handler},
   {"jeq", PT_map_jeq_handler},
   {"return", PT_map_return_handler},
@@ -75,3 +77,19 @@ const struct _map_dictionary_line bta_map_dictionary[] = {
   {"ip", PT_map_ip_handler},
   {"", NULL}
 };
+
+const struct _map_dictionary_line sampling_map_dictionary[] = {
+  {"id", PT_map_id_handler},
+  {"ip", PT_map_ip_handler},
+  {"in", PT_map_input_handler},
+  {"out", PT_map_output_handler},
+  {"", NULL}
+};
+
+const struct _map_dictionary_line bitr_map_dictionary[] = {
+  {"id", PT_map_id_handler},
+  {"ip", PT_map_ip_handler},
+  {"in", PT_map_input_handler},
+  {"out", PT_map_output_handler},
+  {"", NULL}
+};
