@@ -1,6 +1,6 @@
 /* Copyright (C) InMon Corporation 2002-2003 ALL RIGHTS RESERVED */
 
-/* $Header: /home/repo-0.12/pmacct/src/sflow.h,v 1.1.1.1 2007/02/24 23:00:05 paolo Exp $ */
+/* $Header: /home/repo-0.14/pmacct/src/sflow.h,v 1.2 2012/02/07 20:51:59 paolo Exp $ */
 
 /////////////////////////////////////////////////////////////////////////////////
 /////////////////////// sFlow Sampling Packet Data Types ////////////////////////
@@ -325,12 +325,11 @@ typedef struct _SFLFlow_sample_element {
   SFLFlow_type flowType;
 } SFLFlow_sample_element;
 
-enum SFL_sample_tag {
-  SFLFLOW_SAMPLE = 1,              /* enterprise = 0 : format = 1 */
-  SFLCOUNTERS_SAMPLE = 2,          /* enterprise = 0 : format = 2 */
-  SFLFLOW_SAMPLE_EXPANDED = 3,     /* enterprise = 0 : format = 3 */
-  SFLCOUNTERS_SAMPLE_EXPANDED = 4  /* enterprise = 0 : format = 4 */
-};
+#define SFLFLOW_SAMPLE 1               /* enterprise = 0 : format = 1 */
+#define SFLCOUNTERS_SAMPLE 2           /* enterprise = 0 : format = 2 */
+#define SFLFLOW_SAMPLE_EXPANDED 3      /* enterprise = 0 : format = 3 */
+#define SFLCOUNTERS_SAMPLE_EXPANDED 4  /* enterprise = 0 : format = 4 */
+#define SFLACL_BROCADE_SAMPLE 8155137  /* enterprise = 1991 : format = 1 */
   
 /* Format of a single flow sample */
 
