@@ -1,6 +1,6 @@
 /*
     pmacct (Promiscuous mode IP Accounting package)
-    pmacct is Copyright (C) 2003-2010 by Paolo Lucente
+    pmacct is Copyright (C) 2003-2011 by Paolo Lucente
 */
 
 /*
@@ -50,12 +50,15 @@ void sqlite3_plugin(int pipe_fd, struct configuration *cfgptr, void *ptr)
   int pollagain = TRUE;
   u_int32_t seq = 1, rg_err_count = 0; 
 
-  /* XXX: glue */
   memcpy(&config, cfgptr, sizeof(struct configuration));
   recollect_pipe_memory(ptr);
   pm_setproctitle("%s [%s]", "SQLite3 Plugin", config.name);
   memset(&idata, 0, sizeof(idata));
   if (config.pidfile) write_pid_file_plugin(config.pidfile, config.type, config.name);
+  if (config.logfile) {
+    fclose(config.logfile_fd);
+    config.logfile_fd = open_logfile(config.logfile);
+  }
 
   sql_set_signals();
   sql_init_default_values();
@@ -100,6 +103,19 @@ void sqlite3_plugin(int pipe_fd, struct configuration *cfgptr, void *ptr)
 
     idata.now = time(NULL);
 
+    if (config.sql_history) {
+      while (idata.now > (idata.basetime + idata.timeslot)) {
+        time_t saved_basetime = idata.basetime;
+
+        idata.basetime += idata.timeslot;
+        if (config.sql_history == COUNT_MONTHLY)
+          idata.timeslot = calc_monthly_timeslot(idata.basetime, config.sql_history_howmany, ADD);
+        glob_basetime = idata.basetime;
+        idata.new_basetime = saved_basetime;
+        glob_new_basetime = saved_basetime;
+      }
+    }
+
     switch (ret) {
     case 0: /* timeout */
       if (qq_ptr) sql_cache_flush(queries_queue, qq_ptr, &idata, FALSE);
@@ -240,18 +256,6 @@ void sqlite3_plugin(int pipe_fd, struct configuration *cfgptr, void *ptr)
       }
 
       data = (struct pkt_data *) (pipebuf+sizeof(struct ch_buf_hdr));
-      if (config.sql_history) {
-        while (idata.now > (idata.basetime + idata.timeslot)) {
-	  time_t saved_basetime = idata.basetime;
-
-	  idata.basetime += idata.timeslot;
-	  if (config.sql_history == COUNT_MONTHLY)
-	    idata.timeslot = calc_monthly_timeslot(idata.basetime, config.sql_history_howmany, ADD);
-	  glob_basetime = idata.basetime;
-	  idata.new_basetime = saved_basetime;
-	  glob_new_basetime = saved_basetime;
-	}
-      }
 
       while (((struct ch_buf_hdr *)pipebuf)->num) {
 	for (num = 0; net_funcs[num]; num++)
@@ -420,11 +424,13 @@ void SQLI_cache_purge(struct db_cache *queue[], int index, struct insert_data *i
   /* We check for variable substitution in SQL table */ 
   if (idata->dyn_table) {
     char tmpbuf[LONGLONGSRVBUFLEN];
+    time_t stamp = idata->new_basetime ? idata->new_basetime : idata->basetime;
 
-    strftime_same(insert_clause, LONGSRVBUFLEN, tmpbuf, &idata->basetime);
-    strftime_same(update_clause, LONGSRVBUFLEN, tmpbuf, &idata->basetime);
-    strftime_same(lock_clause, LONGSRVBUFLEN, tmpbuf, &idata->basetime);
-    if (config.sql_table_schema && idata->new_basetime) sql_create_table(bed.p, idata);
+    strftime_same(insert_clause, LONGSRVBUFLEN, tmpbuf, &stamp);
+    strftime_same(update_clause, LONGSRVBUFLEN, tmpbuf, &stamp);
+    strftime_same(lock_clause, LONGSRVBUFLEN, tmpbuf, &stamp);
+
+    if (config.sql_table_schema) sql_create_table(bed.p, &stamp);
   }
   // strncat(update_clause, set_clause, SPACELEFT(update_clause));
 
